import unittest

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from readchar import key

import inquirer.questions as questions
from inquirer.render import ConsoleRender
from inquirer import errors
from . import helper


class EditorRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    @patch('editor.edit')
    def test_basic_render(self, edit):
        edit.return_value = b'Some text'
        stdin = [key.ENTER]
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Editor(variable, message)

        sut = ConsoleRender(event_generator=helper.event_factory(*stdin))
        result = sut.render(question)

        self.assertEqual('Some text', result)
        self.assertInStdout(message)
        self.assertTrue(edit.called)

    @patch('editor.edit')
    def test_ignore_true_should_return(self, edit):
        edit.return_value = b'Some text'
        stdin = [key.ENTER]
        message = 'Foo message'
        variable = 'Bar variable'
        expected = 'Something default'

        question = questions.Editor(variable,
                                    ignore=True,
                                    default=expected,
                                    message=message)

        sut = ConsoleRender(event_generator=helper.event_factory(*stdin))
        result = sut.render(question)

        self.assertEqual(expected, result)
        self.assertNotInStdout(message)
        self.assertFalse(edit.called)

    @patch('editor.edit')
    def test_validation_fails(self, edit):
        stdin = [key.ENTER, key.ENTER]
        edit.side_effect = [b'Only one line', b'Two\nLines\nCool']

        message = 'Insert number'
        variable = 'foo'
        expected = 'Two\nLines\nCool'

        def val(_, x):
            return x.count('\n') >= 2

        question = questions.Editor(variable,
                                    validate=val,
                                    message=message)

        sut = ConsoleRender(event_generator=helper.event_factory(*stdin))
        result = sut.render(question)
        self.assertEqual(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('Entered value is not a valid foo')
        self.assertTrue(edit.called)

    @patch('editor.edit')
    def test_validation_fails_with_custom_error(self, edit):
        stdin = [key.ENTER, key.ENTER]
        edit.side_effect = [b'Only one line', b'Two\nLines\nCool']

        message = 'Insert number'
        variable = 'foo'
        expected = 'Two\nLines\nCool'

        def val(_, x):
            if x.count('\n') < 2:
                raise errors.ValidationError('', reason='Some bad reason')

            return True

        question = questions.Editor(variable,
                                    validate=val,
                                    message=message)

        sut = ConsoleRender(event_generator=helper.event_factory(*stdin))
        result = sut.render(question)
        self.assertEqual(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('Some bad reason')
        self.assertTrue(edit.called)

    @patch('editor.edit')
    def test_ctrl_c_breaks_execution(self, edit):
        stdin = [key.CTRL_C]
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Editor(variable, message)

        sut = ConsoleRender(event_generator=helper.event_factory(*stdin))
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)
        self.assertFalse(edit.called)



import unittest
try:
    from unittest.mock import MagicMock, Mock
except ImportError:
    from mock import MagicMock, Mock

from inquirer import prompt


class PromptTests(unittest.TestCase):
    def test_prompt_returns_a_hash(self):
        self.assertEqual({}, prompt([]))

    def test_prompt_renders_a_questions(self):
        question1 = MagicMock()
        question1.name = 'foo'
        result1 = object()
        render = Mock()
        render.render.return_value = result1

        result = prompt([question1], render=render)

        self.assertEqual({'foo': result1}, result)
        self.assertTrue(render.render.called)
        render.render.call_args_list[0][0] == result1



import unittest
import pexpect
from readchar import key


expected_result = r"""\
{'correct': True,\r\n
 'organization': '',\r\n
 'password': 'edcba',\r\n
 'repo': 'default',\r\n
 'topics': \['common'\],\r\n
 'user': 'abcde'}\r\n"""


class PreAnswersTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/pre_answers.py')

    def test_minimal_input(self):
        # user
        self.sut.expect("Please enter", timeout=1)
        self.sut.send('abcde')
        self.sut.send(key.ENTER)
        # password
        self.sut.expect("Please enter", timeout=1)
        self.sut.send('edcba')
        self.sut.send(key.ENTER)
        # repo
        self.sut.expect("Please enter", timeout=1)
        self.sut.send(key.ENTER)
        # topics
        self.sut.expect("Please define", timeout=1)
        self.sut.send(key.SPACE)
        self.sut.send(key.ENTER)
        # organization
        self.sut.expect("If this is", timeout=1)
        self.sut.send(key.ENTER)
        # correct
        self.sut.expect("This will delete", timeout=1)
        self.sut.send('y')

        # again

        # user
        self.sut.expect("Please enter", timeout=1)
        self.sut.send(key.ENTER)
        # password
        self.sut.expect("Please enter", timeout=1)
        self.sut.send(key.ENTER)
        # repo
        self.sut.expect("Please enter", timeout=1)
        self.sut.send(key.ENTER)
        # topics
        self.sut.expect("Please define", timeout=1)
        self.sut.send(key.ENTER)
        # organization
        self.sut.expect("If this is", timeout=1)
        self.sut.send(key.ENTER)
        # correct
        self.sut.expect("This will delete", timeout=1)
        self.sut.send(key.ENTER)

        self.sut.expect([expected_result, pexpect.EOF], timeout=1)



# -*- coding: utf-8 -*-

from .render.console import ConsoleRender
from . import themes


def prompt(questions, render=None, answers=None,
           theme=themes.Default(), raise_keyboard_interrupt=False):
    render = render or ConsoleRender(theme=theme)
    answers = answers or {}

    try:
        for question in questions:
            answers[question.name] = render.render(question, answers)
        return answers
    except KeyboardInterrupt:
        if raise_keyboard_interrupt:
            raise
        print('')
        print('Cancelled by user')
        print('')



import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender


class CheckboxRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_all_choices_are_shown(self):
        stdin = helper.event_factory(key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        sut.render(question)

        self.assertInStdout(message)
        for choice in choices:
            self.assertInStdout(choice)

    def test_one_choice(self):
        stdin = helper.event_factory(key.SPACE, key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertInStdout(message)
        self.assertEqual(['foo'], result)

    def test_choose_the_second(self):
        stdin = helper.event_factory(key.DOWN, key.SPACE, key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertInStdout(message)
        self.assertEqual(['bar'], result)

    def test_can_move(self):
        stdin = helper.event_factory(
            key.DOWN,
            key.DOWN,
            key.UP,
            key.SPACE,
            key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(['bar'], result)

    def test_cannot_move_beyond_upper_limit(self):
        stdin = helper.event_factory(
            key.UP,
            key.UP,
            key.UP,
            key.SPACE,
            key.ENTER,)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(['foo'], result)

    def test_cannot_move_beyond_lower_limit(self):
        stdin = helper.event_factory(
            key.DOWN,
            key.DOWN,
            key.DOWN,
            key.DOWN,
            key.DOWN,
            key.DOWN,
            key.DOWN,
            key.SPACE,
            key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.printStdout()

        self.assertEqual(['bazz'], result)

    def test_ctrl_c_breaks_execution(self):
        stdin_array = [key.CTRL_C]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)

    def test_deselection(self):
        stdin_array = [key.SPACE, key.SPACE, key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual([], result)

    def test_right_cursor_selects_too(self):
        stdin_array = [key.RIGHT, key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(['foo'], result)

    def test_right_cursor_do_not_unselect(self):
        stdin_array = [key.RIGHT, key.RIGHT, key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(['foo'], result)

    def test_left_cursor_unselect(self):
        stdin_array = [key.SPACE, key.LEFT, key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual([], result)

    def test_left_cursor_do_not_select(self):
        stdin_array = [key.SPACE, key.LEFT, key.LEFT, key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.Checkbox(variable, message, choices=choices)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual([], result)



import re
import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender
from inquirer import errors


class TextRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_basic_render(self):
        stdin_msg = 'This is a foo message'
        stdin_array = [x for x in stdin_msg + key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(stdin_msg, result)
        self.assertInStdout(message)

    def test_ignore_true_should_return(self):
        stdin = 'This is a foo message'
        message = 'Foo message'
        variable = 'Bar variable'
        expected = ''

        question = questions.Text(variable,
                                  ignore=True,
                                  default=expected,
                                  message=message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual(expected, result)
        self.assertNotInStdout(message)

    def test_validation_fails(self):
        stdin_array = [x for x in
                       'Invalid' + key.ENTER +
                       key.BACKSPACE * 20 +
                       '9999' + key.ENTER]
        stdin = helper.event_factory(*stdin_array)

        message = 'Insert number'
        variable = 'foo'
        expected = '9999'

        question = questions.Text(variable,
                                  validate=lambda _, x: re.match(r'\d+', x),
                                  message=message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)
        self.assertEqual(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('"Invalid" is not a valid foo')

    def test_validation_fails_with_custom_message(self):
        stdin_array = [x for x in
                       'Invalid' + key.ENTER +
                       key.BACKSPACE * 20 +
                       '9999' + key.ENTER]
        stdin = helper.event_factory(*stdin_array)

        message = 'Insert number'
        variable = 'foo'
        expected = '9999'

        def raise_exc(x, current):
            if current != '9999':
                raise errors.ValidationError('', reason='Custom error')
            return True

        question = questions.Text(variable,
                                  validate=raise_exc,
                                  message=message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)
        self.assertEqual(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('Custom error')

    def test_allows_deletion(self):
        stdin_array = ['a', key.BACKSPACE, 'b', key.ENTER]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual('b', result)

    def test_cursor_movement(self):
        stdin_array = [
            'a',
            key.UP,
            'b',
            key.DOWN,
            'c',
            key.LEFT,
            'd',
            key.RIGHT,
            'e',
            key.ENTER,
        ]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(event_generator=stdin)
        result = sut.render(question)

        self.assertEqual('abdce', result)

    def test_ctrl_c_breaks_execution(self):
        stdin_array = [key.CTRL_C]
        stdin = helper.event_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(event_generator=stdin)
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)



import json
import unittest

from blessed import Terminal

from inquirer import themes
from inquirer import errors


class ThemeTests(unittest.TestCase):
    def setUp(self):
        self.term = Terminal()
        self.theme_dict = {
            'Question': {
                'mark_color': 'red',
                'brackets_color': 'yellow'
            },
            'List': {
                'selection_color': 'red',
                'selection_cursor': '->'
            }
        }
        self.theme_dict_wrong_field = {
            'Question': {
                'ark_color': 'red'
            }
        }
        self.theme_dict_wrong_question = {
            'questionn': {
                'mark_color': 'red'
            }
        }

    def test_load_from_dict(self):
        theme = themes.load_theme_from_dict(self.theme_dict)
        assert theme.Question.mark_color in ['red', '\x1b[31m']
        assert theme.Question.brackets_color in ['yellow', '\x1b[33m']
        assert theme.List.selection_color in ['red', '\x1b[31m']
        assert theme.List.selection_cursor == '->'

    def test_load_from_json(self):
        theme = themes.load_theme_from_json(json.dumps(self.theme_dict))
        assert theme.Question.mark_color in ['red', '\x1b[31m']
        assert theme.Question.brackets_color in ['yellow', '\x1b[33m']
        assert theme.List.selection_color in ['red', '\x1b[31m']
        assert theme.List.selection_cursor == '->'

    def test_invalid_question_field(self):
        with self.assertRaises(errors.ThemeError) as error:
            themes.load_theme_from_dict(self.theme_dict_wrong_field)
        assert 'ark_color' in str(error.exception)

    def test_invalid_question(self):
        with self.assertRaises(errors.ThemeError) as error:
            themes.load_theme_from_dict(self.theme_dict_wrong_question)
        assert 'questionn' in str(error.exception)



import sys

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from inquirer import events


class Iterable(object):
    def __init__(self, *args):
        self.iterator = args.__iter__()

    def next(self):
        return events.KeyPressed(next(self.iterator))


def event_factory(*args):
    return Iterable(*args)


class BaseTestCase(object):
    def base_setup(self):
        self._base_stdin = sys.stdin
        self._base_stdout = sys.stdout
        sys.stdin = StringIO()
        sys.stdout = StringIO()

    def base_teardown(self):
        sys.stdin = self._base_stdin
        sys.stdout = self._base_stdout

    def printStdout(self):
        sys.stdout.seek(0)
        self._base_stdout.write(sys.stdout.read())

    def assertInStdout(self, message):
        sys.stdout.seek(0)
        stdout = sys.stdout.read()
        self.assertIn(message, stdout)

    def assertNotInStdout(self, message):
        sys.stdout.seek(0)
        stdout = sys.stdout.read()
        self.assertNotIn(message, stdout)



# -*- coding: utf-8 -*-

from ._text import Text


class Password(Text):
    def get_current_value(self):
        return self.question.echo * len(self.current) + (
            self.terminal.move_left * self.cursor_offset
        )

    def handle_validation_error(self, error):
        if error.reason:
            return error.reason

        return 'Entered value is not a valid {q}.'.format(q=self.question.name)



import unittest
import pexpect
from readchar import key


class ShortcutsTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/shortcuts.py')

    def set_username(self, name='foo'):
        self.sut.expect("Enter your username", timeout=1)
        self.sut.sendline(name)

    def set_password(self, password='secret'):
        self.sut.expect("Please enter your password", timeout=1)
        self.sut.sendline(password)

    def set_checkbox(self):
        self.sut.expect('Please define your type of project', timeout=1)
        self.sut.send(key.ENTER)

    def set_list(self):
        self.sut.expect('Public or private?', timeout=1)
        self.sut.send(key.ENTER)

    def set_confirm(self):
        self.sut.expect('This will delete', timeout=1)
        self.sut.send('y')
        self.sut.send(key.ENTER)

    def test_shortcuts(self):
        self.set_username()
        self.set_password()
        self.set_checkbox()
        self.set_list()
        self.set_confirm()



# encoding: utf-8

import os
import shutil
import tempfile
import unittest

from inquirer import questions
from inquirer import errors


class BaseQuestionTests(unittest.TestCase):
    def test_base_question_type(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertEqual('base question', q.kind)
        self.assertEqual(name, q.name)

    def test_ignore_works_for_true(self):
        name = 'foo'
        q = questions.Question(name, ignore=True)

        self.assertTrue(q.ignore)

    def test_ignore_works_for_false(self):
        name = 'foo'
        q = questions.Question(name, ignore=False)

        self.assertFalse(q.ignore)

    def test_ignore_works_for_function_returning_true(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: True)

        self.assertTrue(q.ignore)

    def test_ignore_works_for_function_returning_false(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: False)

        self.assertFalse(q.ignore)

    def test_ignore_works_for_function_returning_none(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: None)

        self.assertFalse(q.ignore)

    def test_ignore_function_receives_answers(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: isinstance(x, dict))

        self.assertTrue(q.ignore, "Method was not called with a dict instance")

    def test_default_message_is_empty(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertEqual('', q.message)

    def test_message_set(self):
        name = 'foo'
        message = 'bar'
        q = questions.Question(name, message=message)

        self.assertEqual(message, q.message)

    def test_message_previous_answers_replacement(self):
        name = 'foo'
        message = 'replacement == {whatever}'
        expected = 'replacement == replacement'
        q = questions.Question(name, message=message)
        q.answers = {'whatever': 'replacement'}

        self.assertEqual(expected, q.message)

    def test_default_default_value(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertIsNone(q.default)

    def test_setting_default_value(self):
        name = 'foo'
        expected = object()
        q = questions.Question(name, default=expected)

        self.assertEqual(expected, q.default)

    def test_default_choices_value(self):
        name = 'foo'
        expected = []
        q = questions.Question(name)

        self.assertEqual(expected, q.choices)

    def test_setting_choices_value(self):
        name = 'foo'
        expected = [object(), ]
        q = questions.Question(name, choices=expected)

        self.assertEqual(expected, q.choices)

    def test_validate_false_raises_exception(self):
        name = 'foo'
        q = questions.Question(name, validate=False)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_returning_false_raises_exception(self):
        name = 'foo'
        q = questions.Question(name, validate=lambda x, y: False)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_returning_true_ends_ok(self):
        name = 'foo'
        q = questions.Question(name, validate=lambda x, y: True)

        q.validate(None)

    def test_validate_function_raising_exception(self):
        def raise_exc(x, y):
            raise Exception('foo')

        name = 'foo'
        q = questions.Question(name, validate=raise_exc)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_raising_validation_error(self):
        err = errors.ValidationError('', reason='foo')

        def raise_exc(x, y):
            raise err

        name = 'foo'
        q = questions.Question(name, validate=raise_exc)

        try:
            q.validate(None)
        except errors.ValidationError as e:
            self.assertIs(e, err)

    def test_validate_function_receives_object(self):
        expected = object()

        def compare(x, y):
            return expected == y

        name = 'foo'
        q = questions.Question(name, validate=compare)

        try:
            q.validate(expected)
        except errors.ValidationError:
            self.fail('Validation function did not receive the current value')

    def test_factory_text_type(self):
        name = 'foo'
        q = questions.question_factory('text', name)

        self.assertEqual('text', q.kind)
        self.assertIsInstance(q, questions.Text)
        self.assertEqual(name, q.name)

    def test_factory_confirm_type(self):
        name = 'foo'
        q = questions.question_factory('confirm', name)

        self.assertEqual('confirm', q.kind)
        self.assertIsInstance(q, questions.Confirm)
        self.assertEqual(name, q.name)

    def test_factory_password_type(self):
        name = 'foo'
        q = questions.question_factory('password', name)

        self.assertEqual('password', q.kind)
        self.assertIsInstance(q, questions.Password)
        self.assertEqual(name, q.name)

    def test_factory_list_type(self):
        name = 'foo'
        q = questions.question_factory('list', name)

        self.assertEqual('list', q.kind)
        self.assertIsInstance(q, questions.List)
        self.assertEqual(name, q.name)

    def test_factory_located_list_type(self):
        name = 'ñçÑÇ'
        q = questions.question_factory('list', name)

        self.assertEqual('list', q.kind)
        self.assertIsInstance(q, questions.List)
        self.assertEqual(name, q.name)

    def test_factory_checkbox_type(self):
        name = 'foo'
        q = questions.question_factory('checkbox', name)

        self.assertEqual('checkbox', q.kind)
        self.assertIsInstance(q, questions.Checkbox)
        self.assertEqual(name, q.name)

    def test_load_from_dict_text_type(self):
        name = 'foo'
        q = questions.load_from_dict({'kind': 'text', 'name': name})

        self.assertEqual('text', q.kind)
        self.assertIsInstance(q, questions.Text)
        self.assertEqual(name, q.name)

    def test_load_from_json_text_type(self):
        name = 'foo'
        q = questions.load_from_json(
            '{"kind": "text", "name": "%s"}' % name)

        self.assertEqual('text', q.kind)
        self.assertIsInstance(q, questions.Text)
        self.assertEqual(name, q.name)

    def test_factory_bad_type(self):
        name = 'foo'
        with self.assertRaises(errors.UnknownQuestionTypeError):
            questions.question_factory('bad', name)

    def test_load_from_json_list(self):
        name = 'foo'
        result = questions.load_from_json(
            '[{"kind": "text", "name": "%s"}]' % name)

        self.assertIsInstance(result, list)
        self.assertEqual(1, len(result))
        self.assertEqual('text', result[0].kind)
        self.assertIsInstance(result[0], questions.Text)
        self.assertEqual(name, result[0].name)


class TestConfirmQuestion(unittest.TestCase):
    def test_default_default_value_is_false_instead_of_none(self):
        name = 'foo'
        q = questions.Confirm(name)

        self.assertEqual(False, q.default)


class TestPathQuestion(unittest.TestCase):
    def test_path_validation(self):
        def do_test(path, result=True):
            q = questions.Path('validation_test')
            if result:
                self.assertIsNone(q.validate(path))
            else:
                with self.assertRaises(errors.ValidationError):
                    q.validate(path)

        do_test(None, False)

        if os.environ.get('TRAVIS_PYTHON_VERSION') != 'pypy3':
            # Path component must not be longer then 255 bytes
            do_test('a' * 256, False)
            do_test(os.path.abspath('/asdf/' + 'a' * 256), False)
            do_test('{}/{}'.format('a' * 255, 'b' * 255), True)

            # Path component must not contains null bytes
            do_test('some/path/with/{}byte'.format(b'\x00'.decode('utf-8')),
                    False)

    def test_path_type_validation_no_existence_check(self):
        def do_test(path_type, path, result=True):
            q = questions.Path('path_type_test', path_type=path_type)
            if result:
                self.assertIsNone(q.validate(path))
            else:
                with self.assertRaises(errors.ValidationError):
                    q.validate(path)

        do_test(questions.Path.ANY, './aa/bb')
        do_test(questions.Path.ANY, './aa/')
        do_test(questions.Path.ANY, 'aa/bb')
        do_test(questions.Path.ANY, 'aa/')
        do_test(questions.Path.ANY, '/aa/')
        do_test(questions.Path.ANY, '/aa/bb')
        do_test(questions.Path.ANY, '~/aa/bb')

        do_test(questions.Path.FILE, './aa/bb')
        do_test(questions.Path.FILE, './aa/', False)
        do_test(questions.Path.FILE, 'aa/bb')
        do_test(questions.Path.FILE, 'aa/', False)
        do_test(questions.Path.FILE, '/aa/', False)
        do_test(questions.Path.FILE, '~/aa/', False)
        do_test(questions.Path.FILE, '/aa/bb')
        do_test(questions.Path.FILE, '~/aa/.bb')

        do_test(questions.Path.DIRECTORY, './aa/bb', False)
        do_test(questions.Path.DIRECTORY, './aa/')
        do_test(questions.Path.DIRECTORY, 'aa/bb', False)
        do_test(questions.Path.DIRECTORY, 'aa/')
        do_test(questions.Path.DIRECTORY, '/aa/')
        do_test(questions.Path.DIRECTORY, '~/aa/')
        do_test(questions.Path.DIRECTORY, '/aa/bb', False)
        do_test(questions.Path.DIRECTORY, '~/aa/bb', False)

    def test_path_type_validation_existing(self):
        root = tempfile.mkdtemp()
        some_existing_dir = os.path.join(root, 'some_dir')
        some_non_existing_dir = os.path.join(root, 'some_non_existing_dir')
        some_existing_file = os.path.join(root, 'some_file')
        some_non_existing_file = os.path.join(root, 'some_non_existing_file')

        os.mkdir(some_existing_dir)
        open(some_existing_file, 'a').close()

        def do_test(path_type, path, exists, result=True):
            q = questions.Path('path_type_test', exists=exists,
                               path_type=path_type)
            if result:
                self.assertIsNone(q.validate(path))
            else:
                with self.assertRaises(errors.ValidationError):
                    q.validate(path)

        try:
            do_test(questions.Path.ANY, some_existing_file, True, True)
            do_test(questions.Path.ANY, some_non_existing_file, True, False)
            do_test(questions.Path.ANY, some_existing_file, False, False)
            do_test(questions.Path.ANY, some_non_existing_file, False, True)
            do_test(questions.Path.ANY, some_existing_dir, True, True)
            do_test(questions.Path.ANY, some_non_existing_dir, True, False)
            do_test(questions.Path.ANY, some_existing_dir, False, False)
            do_test(questions.Path.ANY, some_non_existing_dir, False, True)

            do_test(questions.Path.FILE, some_existing_file, True, True)
            do_test(questions.Path.FILE, some_non_existing_file, True, False)
            do_test(questions.Path.FILE, some_non_existing_file, False, True)
            do_test(questions.Path.FILE, some_existing_file, False, False)

            do_test(questions.Path.DIRECTORY,
                    some_existing_dir, True, True)
            do_test(questions.Path.DIRECTORY,
                    some_non_existing_dir, True, False)
            do_test(questions.Path.DIRECTORY,
                    some_existing_dir, False, False)
            do_test(questions.Path.DIRECTORY,
                    some_non_existing_dir, False, True)

        finally:
            shutil.rmtree(root)

    def test_normalizing_value(self):
        # Expanding Home
        home = os.path.expanduser('~')
        q = questions.Path('home')

        path = '~/some_path/some_file'
        self.assertNotIn(home, path)
        self.assertIn(home, q.normalize_value(path))

        # Normalizing to absolute path
        root = os.path.abspath(__file__).split(os.path.sep)[0]
        q = questions.Path('abs_path', normalize_to_absolute_path=True)
        self.assertEqual(root, q.normalize_value('some/relative/path').split(
            os.path.sep
        )[0])

    def test_default_value_validation(self):

        with self.assertRaises(ValueError):
            questions.Path('path', default='~/.toggl_log',
                           path_type=questions.Path.DIRECTORY)

        questions.Path('path', default='~/.toggl_log')
